# Generated by Django 2.0.7 on 2018-08-01 21:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('music_publisher', '0005_workacknowledgement'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='writerinwork',
            options={'verbose_name_plural': 'Writers in Work'},
        ),
        migrations.AlterField(
            model_name='workacknowledgement',
            name='status',
            field=models.CharField(choices=[('CO', 'Conflict'), ('DU', 'Duplicate'), ('RA', 'Transaction Accepted'), ('SA', 'Registration Accepted'), ('AC', 'Registration Accepted with Changes'), ('RJ', 'Rejected'), ('NP', 'No Participation'), ('RC', 'Claim rejected')], max_length=2),
        ),
        migrations.AlterUniqueTogether(
            name='alternatetitle',
            unique_together={('work', 'title')},
        ),
        migrations.AlterUniqueTogether(
            name='artistinwork',
            unique_together={('work', 'artist')},
        ),
        migrations.AlterUniqueTogether(
            name='writerinwork',
            unique_together={('work', 'writer')},
        ),
    ]
