# Generated by Django 2.0.7 on 2018-07-25 17:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('music_publisher', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='CWRExport',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('nwr_rev', models.CharField(choices=[('NWR', 'New work registration'), ('REV', 'Revision of registered work')], db_index=True, default='NWR', max_length=3, verbose_name='NWR/REV')),
                ('cwr', models.TextField(blank=True, editable=False)),
            ],
            options={
                'verbose_name': 'CWR Export',
            },
        ),
        migrations.AddField(
            model_name='albumcd',
            name='_cwr',
            field=models.BooleanField(default=False, editable=False, verbose_name='CWR-Compliance'),
        ),
        migrations.AddField(
            model_name='alternatetitle',
            name='_cwr',
            field=models.BooleanField(default=False, editable=False, verbose_name='CWR-Compliance'),
        ),
        migrations.AddField(
            model_name='artist',
            name='_cwr',
            field=models.BooleanField(default=False, editable=False, verbose_name='CWR-Compliance'),
        ),
        migrations.AddField(
            model_name='firstrecording',
            name='_cwr',
            field=models.BooleanField(default=False, editable=False, verbose_name='CWR-Compliance'),
        ),
        migrations.AddField(
            model_name='work',
            name='_cwr',
            field=models.BooleanField(default=False, editable=False, verbose_name='CWR-Compliance'),
        ),
        migrations.AddField(
            model_name='writer',
            name='_cwr',
            field=models.BooleanField(default=False, editable=False, verbose_name='CWR-Compliance'),
        ),
        migrations.AddField(
            model_name='cwrexport',
            name='works',
            field=models.ManyToManyField(to='music_publisher.Work'),
        ),
    ]
