# Generated by Django 2.0.7 on 2018-07-25 20:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('music_publisher', '0002_auto_20180725_1719'),
    ]

    operations = [
        migrations.AddField(
            model_name='cwrexport',
            name='num_in_year',
            field=models.PositiveSmallIntegerField(default=0),
        ),
        migrations.AddField(
            model_name='cwrexport',
            name='year',
            field=models.CharField(blank=True, db_index=True, editable=False, max_length=2),
        ),
    ]
