# Generated by Django 2.0.7 on 2018-07-26 13:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('music_publisher', '0003_auto_20180725_2051'),
    ]

    operations = [
        migrations.CreateModel(
            name='ACKImport',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('filename', models.CharField(editable=False, max_length=60)),
                ('society_code', models.CharField(editable=False, max_length=3)),
                ('society_name', models.CharField(editable=False, max_length=45)),
                ('date', models.DateField(editable=False)),
                ('report', models.TextField(editable=False)),
            ],
            options={
                'verbose_name': 'CWR ACK Import',
            },
        ),
    ]
