# Generated by Django 2.0.7 on 2018-07-26 19:38

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('music_publisher', '0004_ackimport'),
    ]

    operations = [
        migrations.CreateModel(
            name='WorkAcknowledgement',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('society_code', models.CharField(choices=[('101', 'SOCAN, Canada'), ('088', 'CMRRA, Canada'), ('058', 'SACEM, France'), ('068', 'SDRM, France'), ('035', 'GEMA, Germany'), ('074', 'SIAE, Italy'), ('052', 'PRS, United Kingdom'), ('044', 'MCPS, United Kingdom'), ('010', 'ASCAP, United States'), ('021', 'BMI, United States'), ('071', 'SESAC Inc., United States'), ('034', 'HFA, United States')], max_length=3, verbose_name='Society')),
                ('date', models.DateField()),
                ('status', models.CharField(choices=[('CO', 'Conflict'), ('DU', 'Duplicate'), ('RA', 'Transaction Accepted'), ('AS', 'Registration Accepted'), ('AC', 'Registration Accepted with Changes'), ('RJ', 'Rejected'), ('NP', 'No Participation'), ('RC', 'Claim rejected')], max_length=2)),
                ('remote_work_id', models.CharField(blank=True, max_length=20)),
                ('work', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='music_publisher.Work')),
            ],
        ),
    ]
